@@ -7,7 +7,7 @@ use Data::UUID;
 use Crypt::CBC;
 use Digest::MD5 qw/md5_hex/;
 use Digest::SHA qw/sha1_hex sha256_hex/;
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 sub new {
     my $class = shift;
@@ -41,10 +41,7 @@ sub register {
         PeerPort => $self->{PeerPort},
         Proto    => $self->{Proto},
     );
-    unless ($sock) {
-      warn "Cannot Register: $@";
-      return -1;
-    }
+    die $@ unless $sock;
 
     my $form = <<EOF;
 Application-Name: $AppName
@@ -86,15 +83,22 @@ EOF
     $form = _gen_header($self, 'REGISTER', $form);
     $sock->send($form);
 
-    my $ret = '';
-    while (<$sock>) {
-        $_ =~ s!\r\n!!g;
-        print "$_\n" if $self->{Debug};
-        $ret  = $1 if $_ =~ /^GNTP\/1.0 -(\w+).*$/;
-        last if length($_) == 0;
+    my $ret = <$sock>;
+    $ret = $1 if $ret =~ /^GNTP\/1.0 (-?\w+).*$/;
+    print "$_\n" if $self->{Debug};
+
+    my $description = 'failed to register';
+    if ($ret ne 'OK') {
+        while (<$sock>) {
+            $_ =~ s!\r\n!!g;
+            print "$_\n" if $self->{Debug};
+            $description  = $1 if $_ =~ /^Error-Description:\s*(.*)$/;
+            last if length($_) == 0;
+        }
     }
     close $sock;
-    return $ret eq 'OK';
+
+    die $description if $ret ne 'OK';
 }
 
 sub notify {
@@ -127,10 +131,7 @@ sub notify {
         PeerPort => $self->{PeerPort},
         Proto    => $self->{Proto},
     );
-    unless ($sock) {
-        warn "Cannot Notify: $@";
-        return -1;
-    }
+    die $@ unless $sock;
 
     my $form;
     $form.=sprintf("Application-Name: %s\r\r\n",$data{AppName});
@@ -161,23 +162,23 @@ sub notify {
 
     $form = _gen_header($self, 'NOTIFY', $form);
     $sock->send($form);
-    my $ret = '';
-    while (<$sock>) {
-        $_ =~ s!\r\n!!g;
-        print "$_\n" if $self->{Debug};
-        $ret  = $1 if $_ =~ /^GNTP\/1.0 -(\w+).*$/;
-        last if length($_) == 0;
+
+    my $ret = <$sock>;
+    $ret = $1 if $ret =~ /^GNTP\/1.0 (-?\w+).*$/;
+    print "$_\n" if $self->{Debug};
+
+    my $description = 'failed to notify';
+    if ($ret ne 'OK') {
+        while (<$sock>) {
+            $_ =~ s!\r\n!!g;
+            print "$_\n" if $self->{Debug};
+            $description  = $1 if $_ =~ /^Error-Description:\s*(.*)$/;
+            last if length($_) == 0;
+        }
     }
-    if ($ret eq 'OK' && $data{CallbackContext} && $data{CallbackContextType} && !$data{CallbackTarget}) {
-        push @{$self->{Callbacks}}, {
-            AppName  => $self->{AppName},
-            Socket   => $sock,
-            Function => $data{CallbackFunction},
-        };
-    } else {
-        close $sock;
-    }
-    return $ret eq 'OK';
+    close $sock;
+
+    die $description if $ret ne 'OK';
 }
 
 sub subscribe {
@@ -198,10 +199,6 @@ sub subscribe {
         Proto    => $self->{Proto},
     );
     die $@ unless $sock;
-    unless ($sock) {
-      warn "Cannot Subscribe: $@";
-      return -1;
-    }
 
     my $form = <<EOF;
 Subscriber-ID: \$(ID)
@@ -214,31 +211,38 @@ EOF
 
     $form = _gen_header($self, 'SUBSCRIBE', $form);
     $sock->send($form);
-    my $ret = '';
-    while (<$sock>) {
-        $_ =~ s!\r\n!!g;
-        print "[$_]\n" if $self->{Debug};
-        $ret  = $1 if $_ =~ /^GNTP\/1.0 -(\w+).*$/;
-        last if length($_) == 0;
+
+    my $ret = <$sock>;
+    $ret = $1 if $ret =~ /^GNTP\/1.0 (-?\w+).*$/;
+    print "$_\n" if $self->{Debug};
+
+    my $description = 'failed to register';
+    if ($ret ne 'OK') {
+        while (<$sock>) {
+            $_ =~ s!\r\n!!g;
+            print "$_\n" if $self->{Debug};
+            $description  = $1 if $_ =~ /^Error-Description:\s*(.*)$/;
+            last if length($_) == 0;
+        }
+        die $description if $ret ne 'OK';
     }
-    close $sock;
 
     $sock = IO::Socket::INET->new(
         LocalPort => $data{Port},
         Proto => 'tcp',
         Listen => 10,
     );
-    unless ($sock) {
-      warn "Cannot Subscribe: $@";
-      return -1;
-    }
+    die $@ unless $sock;
+
+    $description = 'failed to subscribe';
     while (1) {
         my $client = $sock->accept();
         my ($Title, $Message) = ('', '');
         while (<$client>){
             $_ =~ s!\r\n!!g;
             print "$_\n" if $self->{Debug};
-            $ret     = $1 if $_ =~ /^GNTP\/1.0 -(\w+).*/;
+            $ret     = $1 if $_ =~ /^GNTP\/1.0 (\w+).*/;
+            $description  = $1 if $_ =~ /^Error-Description:\s*(.*)$/;
             $Title   = $1 if $_ =~ /^Notification-Title: (.*)\r\n/;
             $Message = $1 if $_ =~ /^Notification-Text: (.*)\r\n/;
             # TODO
@@ -253,7 +257,8 @@ EOF
             $callback->($Title, $Message);
         }
     }
-    return $ret eq 'OK';
+
+    die $description if $ret ne 'OK';
 }
 
 sub wait {
@@ -374,7 +379,7 @@ sub _gen_encrypt {
             my $cbc = Crypt::CBC->new(
                 -key => substr($key, 0, 24),
                 -iv => $iv,
-                -keysize => 24,
+				-keysize => 24,
                 -header => 'none',
                 -literal_key => 1,
                 -padding => 'standard',
